/*
 * SMBResponse.java
 *
 * Created on 02 January 2007, 12:12
 *
 */

package org.apache.nutch.protocol.smb;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import jcifs.smb.SmbException;
import jcifs.smb.SmbFile;
import jcifs.smb.SmbFileInputStream;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.conf.Configuration;
import org.apache.nutch.crawl.CrawlDatum;
import org.apache.nutch.metadata.Metadata;
import org.apache.nutch.net.protocols.Response;
import org.apache.nutch.protocol.Content;

/**
 *
 * @author Armel T. Nene
 */
public class SMBResponse {
    
    private String orig;
    private String base;
    private byte[] content;
    private int code;
    private Metadata headers = new Metadata();
    
    private final SMB file;
    private Configuration conf;
    public static final Log LOG = LogFactory.getLog(SMB.class);
    
    /** Creates a new instance of SMBResponse 
     * @param the url, the crawlDatum, the file, configuration  for Nutch
     * @throws SMBException, IOException
     */
    public SMBResponse(URL url, CrawlDatum datum, SMB file, Configuration conf)
    throws SMBException, IOException  {
        this.orig = url.toString();
        this.base = url.toString();
        this.file = file;
        this.conf = conf;
        
        // check if protocol is a window file share protocol
        if(!"smb".equals(url.getProtocol()))
            throw new SMBException("Not an SMB url: "+url);
        
        try {
            this.content = null;
            // for debugging purpose only
//            System.out.println("creating an smb file from the following url: "+url.toString());
            SmbFile f = new SmbFile(url);
            
//            // for debugging purpose only
//            boolean b = f.exists();
//            System.out.println("Connected to: "+f.getName()+" successfully: "+b);
//            SmbFile[] list = f.listFiles();
//            
//            for (int i = 0; i <list.length; i++){
//                SmbFile s = list[i];
//                System.out.println("Reading the following files: "+s.getName());
//                LOG.debug("Reading the following files or directories: "+s.getName());
//            }
            // end of debugging process
            
            // check to make sure the file or directory exists
            if(!f.exists()){
                LOG.info("SMB file: "+f.getName()+" does not exist");
                this.code = 404; // http Not Found
                return;
            }
            
            if(!f.canRead()){
                LOG.info("Cannot read SMB file: "+f.getName());
                this.code = 401;  // http Unauthorized
                return;
            }
            
            if (f.isDirectory()){
                LOG.info("Reading content of SMB directory: "+f.getName());
                getDirAsHttpResponse(f);
            } else if (f.isFile()){
                LOG.info("Reading content of SMB file: "+f.getName());
                getFileAsHttpResponse(f);
            } else {
                LOG.warn("Internal Server error occured for file: "+f.getName());
                this.code = 500;  // http Internal Server Error
            }
        } catch (Exception ex){
            LOG.fatal("Could not read content of protocol: "+url.toString(), ex);
        }
    }
    
    
    /** 
     * Returns the response code.
     * @return code - the response code 
     */
    public int getCode() { return code; }
    
    /** 
     * Returns the value of a named header. 
     * @return header - the header to be retrieve
     */
    public String getHeader(String name) {
        return headers.get(name);
    }
    
    /**
     * Returns the content of the url
     * @return content - the content of the url
     */
    public byte[] getContent() { return content; }
    
    
    /**
     * Creates a content base on the URL and returns the content
     * @return Content - the new content based on URL
     */
    public Content toContent() {
        return new Content(orig, base, content,
                getHeader(Response.CONTENT_TYPE),
                headers, this.conf);
    }
    
    /**
     * retrieve the content of a directory as html page and set the code as an
     * http response code with a value of 200
     * @param file - the SmbFile to retrieve content
     */
    private void getDirAsHttpResponse(SmbFile f) throws SmbException {
        String path = f.getName();
        this.content = list2html(f.listFiles(), path, "/".equals(path) ? false : true);
        
        // set headers
        headers.set(Response.CONTENT_TYPE, "text/html");
        headers.set(Response.CONTENT_LENGTH, new Integer(this.content.length).toString());
        headers.set(Response.LAST_MODIFIED, new Long(f.lastModified()).toString());
        
        // response code
        this.code =  200;  // http OK
    }
    
    /**
     * retrieve the content of a file and set the code as an
     * http response code with a value of 200
     * @param file - the SmbFile to retrieve content
     */
    private void getFileAsHttpResponse(SmbFile f) throws SmbException, SMBException, IOException {
        long size = f.length();
        if(size > Integer.MAX_VALUE)
            throw new SMBException("file is too large, size: "+size);
        
        // capture content
        int len = (int)size;
        byte[] bytes = new byte[len];
        java.io.InputStream is = new SmbFileInputStream(f);
        int offset = 0;
        int n = 0;
        while(offset < size && (n = is.read(this.content, offset, len-offset)) >= 0){
            offset += n;
        }
        
        if(offset < len){
            if(LOG.isWarnEnabled())
                LOG.warn("not enough bytes read from file: "+f.getCanonicalPath());
        }
        
        is.close();
        
        // set headers
        headers.set(Response.CONTENT_LENGTH, new Long(size).toString());
        headers.set(Response.LAST_MODIFIED, String.valueOf(f.lastModified()));
        headers.set(Response.CONTENT_TYPE, "");
        
        // response code
        this.code = 200; // http OK
    }
    
    /**
     * This method generate an html file from a list of SmbFile
     * @param the smbFile, path, boolean value to include dotdot in the file path
     * @return html content as byte
     */
    private byte[] list2html(SmbFile[] smbFile, String path, boolean includeDotDot) throws SmbException {
        StringBuffer x = new StringBuffer("<html><head>");
        x.append("<title>Index of "+path+"</title></head>\n");
        x.append("<body><h1>Index of "+path+"</h1></body>");
        
        if(includeDotDot){
            x.append("<a href='../'>../</a>\t-\t-\n");
        }
        
        SmbFile f;
        for(int i=0; i<smbFile.length; i++){
            f = smbFile[i];
            String name = f.getName();
            String time = String.valueOf(f.lastModified());
            if(f.isDirectory()){
                x.append("<a href='"+name+"'>"+name+"/</a>\t");
                x.append(time+"\t-\n");
            } else{} // ignore any other
        }
        x.append("</pre></body></html>\n");
        return new String(x).getBytes();
    }
}
